@@ -27,11 +27,13 @@ from PyQt5.Qt import (
 
 ROOT = QModelIndex()
 
+
 class AdaptSQP(SearchQueryParser):
 
     def __init__(self, *args, **kwargs):
         pass
 
+
 class Delegate(QStyledItemDelegate):  # {{{
 
     def __init__(self, view):
@@ -47,6 +49,7 @@ class Delegate(QStyledItemDelegate):  # {{{
 
 # }}}
 
+
 class Tweak(object):  # {{{
 
     def __init__(self, name, doc, var_names, defaults, custom):
@@ -106,6 +109,7 @@ class Tweak(object):  # {{{
 
 # }}}
 
+
 class Tweaks(QAbstractListModel, AdaptSQP):  # {{{
 
     def __init__(self, parent=None):
@@ -298,6 +302,7 @@ class Tweaks(QAbstractListModel, AdaptSQP):  # {{{
 
 # }}}
 
+
 class PluginTweaks(QDialog):  # {{{
 
     def __init__(self, raw, parent=None):
@@ -324,6 +329,7 @@ class PluginTweaks(QDialog):  # {{{
 
 # }}}
 
+
 class TweaksView(QListView):
 
     current_changed = pyqtSignal(object, object)
@@ -340,6 +346,7 @@ class TweaksView(QListView):
         QListView.currentChanged(self, cur, prev)
         self.current_changed.emit(cur, prev)
 
+
 class ConfigWidget(ConfigWidgetBase):
 
     def setupUi(self, x):
@@ -500,7 +507,7 @@ class ConfigWidget(ConfigWidgetBase):
                 import traceback
                 error_dialog(self.gui, _('Failed'),
                         _('There was a syntax error in your tweak. Click '
-                            'the show details button for details.'),
+                            'the "Show details" button for details.'),
                         det_msg=traceback.format_exc(), show=True)
                 return
             self.tweaks.update_tweak(idx, l)
@@ -568,5 +575,3 @@ if __name__ == '__main__':
     # Tweaks()
     # test_widget
     test_widget('Advanced', 'Tweaks')
-
-
