@@ -7,7 +7,7 @@ __docformat__ = 'restructuredtext en'
 Command line interface to the calibre database.
 '''
 
-import sys, os, cStringIO, re
+import cStringIO, csv, os, re, sys
 import unicodedata
 from textwrap import TextWrapper
 from optparse import OptionValueError, OptionGroup
@@ -30,6 +30,8 @@ FIELDS = {
 }
 
 do_notify = True
+
+
 def send_message(msg=''):
     global do_notify
     if not do_notify:
@@ -43,10 +45,12 @@ def send_message(msg=''):
         t.conn.send('refreshdb:'+msg)
         t.conn.close()
 
+
 def write_dirtied(db):
     prints('Backing up metadata')
     db.dump_metadata()
 
+
 def get_parser(usage):
     parser = OptionParser(usage)
     go = parser.add_option_group(_('GLOBAL OPTIONS'))
@@ -60,6 +64,7 @@ def get_parser(usage):
 
     return parser
 
+
 def get_db(dbpath, options):
     global do_notify
     if options.library_path is not None:
@@ -72,6 +77,7 @@ def get_db(dbpath, options):
         do_notify = False
     return LibraryDatabase(dbpath)
 
+
 def do_list(db, fields, afields, sort_by, ascending, search_text, line_width, separator,
             prefix, limit, for_machine=False):
     from calibre.utils.terminal import ColoredStream, geometry
@@ -89,6 +95,7 @@ def do_list(db, fields, afields, sort_by, ascending, search_text, line_width, se
         pass
     fields = ['id'] + fields
     title_fields = fields
+
     def field_name(f):
         ans = f
         if f[0] == '*':
@@ -129,8 +136,10 @@ def do_list(db, fields, afields, sort_by, ascending, search_text, line_width, se
             else:
                 record[f] = unicode(record[f])
             record[f] = record[f].replace('\n', ' ')
+
     def chr_width(x):
         return 1 + unicodedata.east_asian_width(x).startswith('W')
+
     def str_width(x):
         return sum(map(chr_width, x))
 
@@ -176,6 +185,7 @@ def do_list(db, fields, afields, sort_by, ascending, search_text, line_width, se
             print >>o
     return o.getvalue()
 
+
 def list_option_parser(db=None):
     fields = set(FIELDS) | {'id'}
     if db is not None:
@@ -257,8 +267,11 @@ class DevNull(object):
 
     def write(self, msg):
         pass
+
+
 NULL = DevNull()
 
+
 def do_add(db, paths, one_book_per_directory, recurse, add_duplicates, otitle,
         oauthors, oisbn, otags, oseries, oseries_index, ocover, oidentifiers, olanguages, compiled_rules):
     orig = sys.stdout
@@ -299,6 +312,7 @@ def do_add(db, paths, one_book_per_directory, recurse, add_duplicates, otitle,
                 mi.series_index = oseries_index
             if ocover:
                 mi.cover = ocover
+                mi.cover_data = (None, None)
 
             formats.append(format)
             metadata.append(mi)
@@ -359,6 +373,7 @@ def do_add(db, paths, one_book_per_directory, recurse, add_duplicates, otitle,
     finally:
         sys.stdout = orig
 
+
 def add_option_parser():
     parser = get_parser(_(
 '''\
@@ -393,6 +408,7 @@ the directory related options below.
 
     g = OptionGroup(parser, _('ADDING FROM DIRECTORIES'), _(
         'Options to control the adding of books from directories. By default only files that have extensions of known e-book file types are added.'))
+
     def filter_pat(option, opt, value, parser, action):
         try:
             getattr(parser.values, option.dest).append(compile_rule({'match_type':'glob', 'query':value, 'action':action}))
@@ -403,6 +419,7 @@ the directory related options below.
                       help=_('Assume that each directory has only a single logical book and that all files in it are different e-book formats of that book'))
     g.add_option('-r', '--recurse', action='store_true', default=False,
                       help=_('Process directories recursively'))
+
     def fadd(opt, action, help):
         g.add_option(
             opt, action='callback', type='string', nargs=1, default=[],
@@ -415,11 +432,12 @@ the directory related options below.
         ' Can be specified multiple times for multiple patterns. For e.g.: *.pdf will ignore all pdf files'))
     fadd('--add', 'add', _(
         'A filename (glob) pattern, files matching this pattern will be added when scanning directories for files,'
-        ' even if they are not of a known ebook file type. Can be specified multiple times for multiple patterns.'))
+        ' even if they are not of a known e-book file type. Can be specified multiple times for multiple patterns.'))
     parser.add_option_group(g)
 
     return parser
 
+
 def do_add_empty(db, title, authors, isbn, tags, series, series_index, cover, identifiers, languages):
     from calibre.ebooks.metadata import MetaInformation
     mi = MetaInformation(None)
@@ -444,6 +462,7 @@ def do_add_empty(db, title, authors, isbn, tags, series, series_index, cover, id
     prints(_('Added book ids: %s')%book_id)
     send_message()
 
+
 def command_add(args, dbpath):
     from calibre.ebooks.metadata import string_to_authors
     parser = add_option_parser()
@@ -468,6 +487,7 @@ def command_add(args, dbpath):
             tags, opts.series, opts.series_index, opts.cover, identifiers, lcodes, opts.filters)
     return 0
 
+
 def do_remove(db, ids):
     book_ids = set()
     for x in ids:
@@ -482,6 +502,7 @@ def do_remove(db, ids):
     from calibre.db.delete_service import delete_service
     delete_service().wait()
 
+
 def remove_option_parser():
     return get_parser(_(
 '''\
@@ -493,6 +514,7 @@ list of id numbers (you can get id numbers by using the search command). For exa
 included).
 '''))
 
+
 def command_remove(args, dbpath):
     parser = remove_option_parser()
     opts, args = parser.parse_args(sys.argv[:1] + args)
@@ -514,6 +536,7 @@ def command_remove(args, dbpath):
 
     return 0
 
+
 def do_add_format(db, id, fmt, path, opts):
     done = db.add_format_with_hooks(id, fmt.upper(), path, index_is_id=True,
                              replace=opts.replace)
@@ -522,12 +545,13 @@ def do_add_format(db, id, fmt, path, opts):
     else:
         send_message()
 
+
 def add_format_option_parser():
     parser = get_parser(_(
 '''\
 %prog add_format [options] id ebook_file
 
-Add the ebook in ebook_file to the available formats for the logical book identified \
+Add the e-book in ebook_file to the available formats for the logical book identified \
 by id. You can get id by using the search command. If the format already exists, \
 it is replaced, unless the do not replace option is specified.\
 '''))
@@ -542,21 +566,23 @@ def command_add_format(args, dbpath):
     if len(args) < 3:
         parser.print_help()
         print
-        print >>sys.stderr, _('You must specify an id and an ebook file')
+        print >>sys.stderr, _('You must specify an id and an e-book file')
         return 1
 
     id, path, fmt = int(args[1]), args[2], os.path.splitext(args[2])[-1]
     if not fmt:
-        print _('ebook file must have an extension')
+        print _('e-book file must have an extension')
     do_add_format(get_db(dbpath, opts), id, fmt[1:], path, opts)
     return 0
 
+
 def do_remove_format(db, id, fmt):
     db.remove_format(id, fmt, index_is_id=True)
     send_message()
     from calibre.db.delete_service import delete_service
     delete_service().wait()
 
+
 def remove_format_option_parser():
     return get_parser(_(
 '''
@@ -582,6 +608,7 @@ def command_remove_format(args, dbpath):
     do_remove_format(get_db(dbpath, opts), id, fmt)
     return 0
 
+
 def do_show_metadata(db, id, as_opf):
     if not db.has_id(id):
         raise ValueError('Id #%d is not present in database.'%id)
@@ -592,6 +619,7 @@ def do_show_metadata(db, id, as_opf):
     else:
         prints(unicode(mi))
 
+
 def show_metadata_option_parser():
     parser = get_parser(_(
 '''
@@ -604,6 +632,7 @@ id is an id number from the search command.
                       help=_('Print metadata in OPF form (XML)'))
     return parser
 
+
 def command_show_metadata(args, dbpath):
     parser = show_metadata_option_parser()
     opts, args = parser.parse_args(sys.argv[1:]+args)
@@ -616,10 +645,12 @@ def command_show_metadata(args, dbpath):
     do_show_metadata(get_db(dbpath, opts), id, opts.as_opf)
     return 0
 
+
 def do_set_metadata(db, id, stream):
     mi = OPF(stream).to_book_metadata()
     db.set_metadata(id, mi)
 
+
 def set_metadata_option_parser():
     parser = get_parser(_(
 '''
@@ -647,6 +678,7 @@ an OPF file.
         ' with the --field option'))
     return parser
 
+
 def embed_metadata_option_parser():
     parser = get_parser(_(
 '''
@@ -664,6 +696,7 @@ separated by hyphens. For example: %prog embed_metadata 1 2 10-15 23'''))
         ' times for multiple formats. By default, all formats are updated.'))
     return parser
 
+
 def command_embed_metadata(args, dbpath):
     parser = embed_metadata_option_parser()
     opts, args = parser.parse_args(sys.argv[0:1]+args)
@@ -679,11 +712,13 @@ def command_embed_metadata(args, dbpath):
         else:
             ids |= {x for x in xrange(int(parts[0], int(parts[1])))}
     only_fmts = opts.only_formats or None
+
     def progress(i, total, mi):
         prints(_('Processed {0} ({1} of {2})').format(mi.title, i, total))
     db.new_api.embed_metadata(ids, only_fmts=only_fmts, report_progress=progress)
     send_message()
 
+
 def command_set_metadata(args, dbpath):
     parser = set_metadata_option_parser()
     opts, args = parser.parse_args(sys.argv[0:1]+args)
@@ -780,6 +815,7 @@ def command_set_metadata(args, dbpath):
 
     return 0
 
+
 def do_export(db, ids, dir, opts):
     if ids is None:
         ids = list(db.all_ids())
@@ -793,6 +829,7 @@ def do_export(db, ids, dir, opts):
             prints('\t'+'\n\t'.join(tb.splitlines()))
             prints(' ')
 
+
 def export_option_parser():
     parser = get_parser(_('''\
 %prog export [options] ids
@@ -830,6 +867,7 @@ an opf file). You can get id numbers from the search command.
 
     return parser
 
+
 def command_export(args, dbpath):
     parser = export_option_parser()
     opts, args = parser.parse_args(sys.argv[1:]+args)
@@ -843,10 +881,12 @@ def command_export(args, dbpath):
     do_export(get_db(dbpath, opts), ids, dir, opts)
     return 0
 
+
 def do_add_custom_column(db, label, name, datatype, is_multiple, display):
     num = db.create_custom_column(label, name, datatype, is_multiple, display=display)
     prints('Custom column created with id: %s'%num)
 
+
 def add_custom_column_option_parser():
     from calibre.library.custom_columns import CustomColumns
     parser = get_parser(_('''\
@@ -900,6 +940,7 @@ def command_add_custom_column(args, dbpath):
     db.prefs['field_metadata'] = db.field_metadata.all_metadata()
     return 0
 
+
 def catalog_option_parser(args):
     from calibre.customize.ui import available_catalog_formats, plugin_for_catalog_format
     from calibre.utils.logging import Log
@@ -951,7 +992,7 @@ def catalog_option_parser(args):
     log = Log()
     parser = get_parser(_(
     '''
-    %prog catalog /path/to/destination.(CSV|EPUB|MOBI|XML ...) [options]
+    %prog catalog /path/to/destination.(csv|epub|mobi|xml...) [options]
 
     Export a catalog in format specified by path/to/destination extension.
     Options control how entries are displayed in the generated catalog output.
@@ -980,6 +1021,7 @@ def catalog_option_parser(args):
 
     return parser, plugin, log
 
+
 def command_catalog(args, dbpath):
     parser, plugin, log = catalog_option_parser(args)
     opts, args = parser.parse_args(sys.argv[1:])
@@ -1006,6 +1048,7 @@ def command_catalog(args, dbpath):
     with plugin:
         return int(bool(plugin.run(args[1], opts, get_db(dbpath, opts))))
 
+
 def parse_series_string(db, label, value):
     val = unicode(value).strip()
     s_index = None
@@ -1023,6 +1066,7 @@ def parse_series_string(db, label, value):
             s_index = 1.0
     return val, s_index
 
+
 def do_set_custom(db, col, id_, val, append):
     if id_ not in db.all_ids():
         prints(_('No book with id: %s in the database')%id_, file=sys.stderr)
@@ -1036,6 +1080,9 @@ def do_set_custom(db, col, id_, val, append):
     else:
         db.set_custom(id_, val, label=col, append=append)
         prints('Data set to: %r'%db.get_custom(id_, label=col, index_is_id=True))
+    write_dirtied(db)
+    send_message()
+
 
 def set_custom_option_parser():
     parser = get_parser(_(
@@ -1066,6 +1113,7 @@ def command_set_custom(args, dbpath):
             opts.append)
     return 0
 
+
 def do_custom_columns(db, details):
     from pprint import pformat
     cols = db.custom_column_label_map
@@ -1078,6 +1126,7 @@ def do_custom_columns(db, details):
         else:
             prints(col, '(%d)'%data['num'])
 
+
 def custom_columns_option_parser():
     parser = get_parser(_(
     '''
@@ -1096,6 +1145,7 @@ def command_custom_columns(args, dbpath):
     do_custom_columns(get_db(dbpath, opts), opts.details)
     return 0
 
+
 def do_remove_custom_column(db, label, force):
     if not force:
         q = raw_input(_('You will lose all data in the column: %s.'
@@ -1110,6 +1160,7 @@ def do_remove_custom_column(db, label, force):
         raise SystemExit(1)
     prints('Column %r removed.'%label)
 
+
 def remove_custom_column_option_parser():
     parser = get_parser(_(
     '''
@@ -1138,6 +1189,7 @@ def command_remove_custom_column(args, dbpath):
     db.prefs['field_metadata'] = db.field_metadata.all_metadata()
     return 0
 
+
 def saved_searches_option_parser():
     parser = get_parser(_(
     '''
@@ -1157,6 +1209,7 @@ def saved_searches_option_parser():
     '''))
     return parser
 
+
 def command_saved_searches(args, dbpath):
     parser = saved_searches_option_parser()
     opts, args = parser.parse_args(args)
@@ -1196,6 +1249,7 @@ def command_saved_searches(args, dbpath):
 
     return 0
 
+
 def backup_metadata_option_parser():
     parser = get_parser(_('''\
 %prog backup_metadata [options]
@@ -1213,6 +1267,7 @@ automatically, every time metadata is changed.
             ' books.'))
     return parser
 
+
 class BackupProgress(object):
 
     def __init__(self):
@@ -1227,6 +1282,7 @@ class BackupProgress(object):
             prints(u'%.1f%% %s - %s'%((self.count*100)/float(self.total),
                 book_id, getattr(mi, 'title', 'Unknown')))
 
+
 def command_backup_metadata(args, dbpath):
     parser = backup_metadata_option_parser()
     opts, args = parser.parse_args(args)
@@ -1269,6 +1325,7 @@ Perform some checks on the filesystem representing a library. Reports are {0}
                              "Default: all"))
     return parser
 
+
 def command_check_library(args, dbpath):
     from calibre.library.check_library import CheckLibrary, CHECKS
     parser = check_library_option_parser()
@@ -1307,28 +1364,35 @@ def command_check_library(args, dbpath):
     else:
         exts = [f.strip() for f in opts.exts.split(',') if f.strip()]
 
-    def print_one(checker, check):
-        attr = check[0]
-        list = getattr(checker, attr, None)
-        if list is None:
-            return
-        if opts.csv:
-            for i in list:
-                print check[1] + ',' + i[0] + ',' + i[1]
-        else:
-            print check[1]
-            for i in list:
-                print '    %-40.40s - %-40.40s'%(i[0], i[1])
-
     if not LibraryDatabase.exists_at(dbpath):
         prints('No library found at', dbpath, file=sys.stderr)
         raise SystemExit(1)
 
     db = LibraryDatabase(dbpath)
+    print _('Vacuuming database...')
+    db.new_api.vacuum()
     checker = CheckLibrary(dbpath, db)
     checker.scan_library(names, exts)
     for check in checks:
-        print_one(checker, check)
+        _print_check_library_results(checker, check, as_csv=opts.csv)
+
+
+def _print_check_library_results(checker, check, as_csv=False, out=sys.stdout):
+    attr = check[0]
+    list = getattr(checker, attr, None)
+    if list is None:
+        return
+
+    if as_csv:
+        to_output = [(check[1], i[0], i[1]) for i in list]
+        csv_print = csv.writer(out)
+        for line in to_output:
+            csv_print.writerow(line)
+
+    else:
+        print >>out, check[1]
+        for i in list:
+            print >>out, '    %-40.40s - %-40.40s'%(i[0], i[1])
 
 
 def restore_database_option_parser():
@@ -1351,6 +1415,7 @@ what is found in the OPF files.
                    'unless this option is specified.'))
     return parser
 
+
 def command_restore_database(args, dbpath):
     parser = restore_database_option_parser()
     opts, args = parser.parse_args(args)
@@ -1398,6 +1463,7 @@ def command_restore_database(args, dbpath):
             prints('Some errors occurred. A detailed report was '
                     'saved to', name)
 
+
 def list_categories_option_parser():
     parser = get_parser(_('''\
 %prog list_categories [options]
@@ -1425,6 +1491,7 @@ information is the equivalent of what is shown in the tags pane.
                              'Default is a comma.'))
     return parser
 
+
 def command_list_categories(args, dbpath):
     parser = list_categories_option_parser()
     opts, args = parser.parse_args(args)
@@ -1524,6 +1591,7 @@ def command_list_categories(args, dbpath):
 
     return parser
 
+
 def clone_option_parser():
     return get_parser(_(
     '''\
@@ -1536,6 +1604,7 @@ The cloned library will contain no books. If you want to create a full duplicate
 all books, then simply use your filesystem tools to copy the library folder.
     '''))
 
+
 def command_clone(args, dbpath):
     parser = clone_option_parser()
     opts, args = parser.parse_args(args)
@@ -1565,6 +1634,7 @@ def command_clone(args, dbpath):
     db.close()
     LibraryDatabase(loc, default_prefs=dbprefs)
 
+
 def search_option_parser():
     parser = get_parser(_(
     '''\
@@ -1581,6 +1651,7 @@ language, for example: {0}
         help=_('The maximum number of results to return. Default is all results.'))
     return parser
 
+
 COMMANDS = ('list', 'add', 'remove', 'add_format', 'remove_format',
             'show_metadata', 'set_metadata', 'export', 'catalog',
             'saved_searches', 'add_custom_column', 'custom_columns',
@@ -1588,6 +1659,7 @@ COMMANDS = ('list', 'add', 'remove', 'add_format', 'remove_format',
             'check_library', 'list_categories', 'backup_metadata',
             'clone', 'embed_metadata', 'search')
 
+
 def command_search(args, dbpath):
     parser = search_option_parser()
     opts, args = parser.parse_args(args)
@@ -1638,5 +1710,6 @@ def main(args=sys.argv):
 
     return command(args[2:], dbpath)
 
+
 if __name__ == '__main__':
     sys.exit(main())
